"""
jlp
~~~~~~~~~

:copyright: (c) 2013 Canonical Ltd
:license: GPLv2

"""

__name__ = 'jlp'
__version__ = '0.1'
__author__ = 'Martin Mrazik'

import yaml
import os
import logging
import sys

logger = logging.getLogger('jenkins-launchpad-plugin')
stdout_handler = logging.StreamHandler(stream=sys.stdout)
formatter = logging.Formatter('%(levelname)s: %(message)s')
stdout_handler.setFormatter(formatter)
logger.addHandler(stdout_handler)

_json_jenkins = None


def get_json_jenkins():
    global _json_jenkins
    if _json_jenkins is not None:
        return _json_jenkins
    try:
        # The jenkins_proxy_url is an addition. Don't raise an exception
        # if it's not defined, just set it to a default value.
        jenkins_proxy_url = get_config_option('jenkins_proxy_url')
    except KeyError:
        jenkins_proxy_url = ''
    _json_jenkins = JSONJenkins(get_config_option('jenkins_url'),
                                jenkins_proxy_url,
                                get_config_option('jenkins_user'),
                                get_config_option('jenkins_password'))
    return _json_jenkins


_config = None


def load_config():
    global _config
    if _config is not None:
        return _config
    files = [os.path.expanduser('~/.jlp/jlp.config'), 'jlp.config']
    for config_file in files:
        try:
            _config = yaml.safe_load(open(config_file, 'r'))
            return _config
        except IOError:
            pass
    if not _config:
        logger.error('No config file found')
        sys.exit(1)


def get_config_option(name):
    config = load_config()
    return config[name]


def set_log_level(logger, stdout_handler):
    log_levels = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'critical': logging.CRITICAL
    }
    try:
        log_level = log_levels[get_config_option('log_level')]
    except KeyError:
        #if log_level is not in the config default to debug
        log_level = logging.DEBUG
        logger.warning('No log_level found in config or unknown log level '
                       'specified. Defaulting to "debug".')
    logger.setLevel(log_level)
    stdout_handler.setLevel(log_level)


set_log_level(logger, stdout_handler)


from .socketlock import SocketLock
assert SocketLock  # silence pyflakes
from .jsonjenkins import JSONJenkins
assert JSONJenkins  # silence pyflakes
from .branch import Branch
assert Branch  # silence pyflakes
from .bzrrecipe import BzrRecipe
assert BzrRecipe  # silence pyflakes
from .launchpadagent import get_launchpad
assert get_launchpad  # silence pyflakes
from .dputrunner import DputRunner
assert DputRunner
