from argparse import ArgumentParser
import atexit
from jlp import get_launchpad, launchpadutils, logger
import os
from shutil import rmtree


def getMergeCount(branch, merge_type, repo_type):
    """Returns the total number of outstanding merge requests for the given
    type."""
    count = 0
    # launchpadlib is not thread/process safe so we are creating launchpadlib
    # cache in /tmp per process which gets cleaned up at the end
    # see also lp:459418 and lp:1025153
    launchpad_cachedir = os.path.join('/tmp',
                                      str(os.getpid()),
                                      '.launchpadlib')

    # `launchpad_cachedir` is leaked upon unexpected exits
    # adding this cleanup to stop directories filling up `/tmp/`
    atexit.register(rmtree, os.path.join('/tmp',
                                         str(os.getpid())),
                    ignore_errors=True)
    lp_handle = get_launchpad(launchpadlib_dir=launchpad_cachedir)
    for b in branch:
        mps = launchpadutils.get_merge_proposals(lp_handle, b, [merge_type], repo_type)
        logger.debug("[%s] merge proposals: %s" % (b, mps))
        logger.debug("count: %d" % (len(mps)))
        count += len(mps)
    return count


def getMergeProposals():
    parser = ArgumentParser(description="Return count of outstanding merge "
                            "proposals.")
    parser.add_argument("-t", "--merge_type", default="Approved",
                        help="Count only this type of merge proposals.")
    parser.add_argument("-r", "--repo_type", default="auto",
                        help="Repository type [bazaar, git, or auto]")
    parser.add_argument("branch", nargs="+",
                        help="Branch to check merge count.")
    args = vars(parser.parse_args())

    return getMergeCount(**args)
