from argparse import ArgumentParser
import atexit
from jlp import launchpadutils, get_launchpad, logger
import re
import os
from shutil import rmtree


def voteOnMergeProposal():
    parser = ArgumentParser(description="Vote on a Launchpad merge proposal.")
    parser.add_argument('-s', '--status')
    parser.add_argument('-u', '--build-url', required=True,
                        help="URL of the Jenkins job")
    parser.add_argument('-r', '--revision', required=True,
                        help="merge proposal candidate revision")
    parser.add_argument('-b', '--branch',
                        help="Not used but kept for backwards compatibility.")
    parser.add_argument('-p', '--merge-proposal', required=True,
                        help="URL of the merge proposal to update")
    args = vars(parser.parse_args())

    # launchpadlib is not thread/process safe so we are creating launchpadlib
    # cache in /tmp per process which gets cleaned up at the end
    # see also lp:459418 and lp:1025153
    launchpad_cachedir = os.path.join('/tmp',
                                      str(os.getpid()),
                                      '.launchpadlib')

    # `launchpad_cachedir` is leaked upon unexpected exits
    # adding this cleanup to stop directories filling up `/tmp/`
    atexit.register(rmtree, os.path.join('/tmp',
                                         str(os.getpid())),
                    ignore_errors=True)
    lp_handle = get_launchpad(launchpadlib_dir=launchpad_cachedir)
    mp = launchpadutils.get_mp_handle_from_url(lp_handle,
                                               args['merge_proposal'])
    if not mp:
        parser.error('merge proposal related to this branch was not found')

    # this is the status from tests
    overal_status = args['status']
    # by default reason is empty as it is usually just a failed build
    reason = ''

    #override the status to FAILED and set reason if no commit message is set
    match = re.match('^(.*)/job/([^/]*)/.*$', args['build_url'])
    if match:
        jenkins_url = match.group(1)
        jenkins_job = match.group(2)
        if not launchpadutils.is_commit_message_set(mp,
                                                    jenkins_job, jenkins_url):
            overal_status = 'FAILED'
            reason = "No commit message was specified in the merge " + \
                     "proposal. Click on the following link and set the " + \
                     "commit message (if you want a jenkins rebuild you " +\
                     "need to trigger it yourself):\n" + \
                     args['merge_proposal'] + "/+edit-commit-message\n"
            logger.debug('Commit message not set. Failing CI.')
    else:
        logger.debug('Unable to get job name from build_url.' +
                     'Skipping check for empty commit message')

    if overal_status == 'PASSED':
        launchpadutils.approve_mp(mp, args['revision'], args['build_url'])
    else:  # status == False corresponds to NOT 'PASSED'
        launchpadutils.disapprove_mp(mp,
                                     args['revision'],
                                     args['build_url'],
                                     reason)

    return 0
